%%%--------------------------------------------------------------------- 
%%% Copyright (C) 2008  Erik G. Burrows
%%%
%%% This program is free software; you can redistribute it and/or modify
%%% it under the terms of the GNU General Public License as published by
%%% the Free Software Foundation; either version 2 of the License, or
%%% (at your option) any later version.
%%%
%%% This program is distributed in the hope that it will be useful,
%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%% GNU General Public License for more details.
%%%
%%% You should have received a copy of the GNU General Public License
%%% along with this program; if not, write to the Free Software
%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%%
%%%--------------------------------------------------------------------- 
%%% Description module nmea
%%%--------------------------------------------------------------------- 
%%% 
%%% Library of functions helpful in processing NMEA sentences.
%%%
%%%--------------------------------------------------------------------- 
%%% Exports
%%%--------------------------------------------------------------------- 
%%% @spec extract_sentences(In) -> {Sentences, Rest}
%%%
%%% Scan a character string for CRLF. Extract sentences,
%%% and return remainder.
%%%
%%% In = Character array input
%%% Sentences = Array of binary strings extracted from input
%%% Rest = Remainder of input, after sentences are extracted
%%%

-module(nmea).

-export([
	 extract_sentences/1
	 ]).

%% @spec extract_sentences(In) -> {Sentence, Rest}
%%
%% Scan a character string for CRLF. Extract sentences,
%% and return remainder.
%% 
%%
%% In = Character array input
%% Sentences = Array of binary strings extracted from input
%% Rest = Remainder of input, after sentences are extracted
%%
extract_sentences(In) ->
    extract_sentences(In, 1).


%%
%% Internal Functions
%%

extract_sentences(In, Offset) ->
    case Offset =< (length(In) - 1) of
        true ->
            case lists:sublist(In, Offset, 2) =:= "\r\n" of
                true ->
                    {SentenceList, Rest} = lists:split(Offset + 1, In),

		    % Code to fail every so often, probably best to leave commented.
		    %ok = case string:substr(SentenceList, 2, 6) of
		    %    "GPSD,R" -> ok;
		    %    "!AIVDM" -> ok;
		    %    "!AIVDO" -> ok;
		    %    _Else -> error
		    %end,

		    Sentence = list_to_binary(SentenceList),

		    {NextSentence, NextRest} = extract_sentences(Rest, 1),

		    {lists:concat([[Sentence], NextSentence]), NextRest};

                false ->
                    extract_sentences(In, Offset + 1)
            end;

        false ->
            {[], In}
    end.
