-module(subscriptions).

-include_lib("common_records.hrl").


-export([
    new/1,
    subscribe/3,
    unsubscribe/3,
    send/4
    ]).


new(NameList) ->
    populate(NameList, orddict:new()).

subscribe(Subscriber, Name, Db) ->
    Sub = orddict:fetch(Name, Db),
    case ordsets:is_element(Subscriber, Sub) of
	false ->
	    NewSub = ordsets:add_element(Subscriber, Sub),
	    orddict:store(Name, NewSub, Db);
	true ->
	    Db
    end.

unsubscribe(Subscriber, Name, Db) ->
    Sub = orddict:fetch(Name, Db),
    NewSub = ordsets:del_element(Subscriber, Sub),
    NewDb = orddict:store(Name, NewSub, Db),
    NewDb.

send(Messages, OutputModule, OutputName, Db) ->
    Subscriberset = orddict:fetch(OutputName, Db),
    Subscriberlist = ordsets:to_list(Subscriberset),
    send_messages(OutputModule, OutputName, Subscriberlist, Messages).

%
% Internal functions
%

populate([], Db) ->
    Db;

populate([H|T], Db) ->
    NewDb = orddict:store(H, ordsets:new(), Db),
    populate(T, NewDb).


send_messages(_, _, _, []) ->
    ok;

send_messages(OutputModule, OutputName, Subscriberlist, [H|T]) ->
    send_notifications(OutputModule, OutputName, Subscriberlist, H),
    send_messages(OutputModule, OutputName, Subscriberlist, T).


send_notifications(_, _, [], _) ->
    ok;

send_notifications(OutputModule, OutputName, [H|T], Sentence) ->
    {Subscriber, InputName} = H,

    %#message{message=Message} = Sentence,
    %io:format("~p:~p -> ~p:~p: ~p\n", [OutputModule, OutputName, Subscriber, InputName, Message]),

    catch gen_server:call({global, Subscriber}, {input, InputName, OutputModule, OutputName, Sentence}),
	
    send_notifications(OutputModule, OutputName, T, Sentence).


